#include <iostream>
#include <fstream>
#include <vector>
#include <TFile.h>
#include <TTree.h>
#include <TH1D.h>
#include "Event.h"
#include <TMVA/Reader.h>

using namespace std;

int main() {

// Set up an output file and book some histograms

  TFile* histFile = new TFile("analysis.root", "RECREATE");
  TH1D* hFishSig = new TH1D("hFishSig", "Fisher, signal", 100, -10.0, 10.0);
  TH1D* hFishBkg = new TH1D("hFishBkg", "Fisher, background", 100, -10.0, 10.0);

// Set up the TMVA Reader object.
// The names in AddVariable must be same as in the input (weight) file.
// x[0] is event weight.

  TMVA::Reader* reader = new TMVA::Reader();
  const int numVar = 12;
  float x[numVar];         // TMVA needs floats here

  reader->AddVariable("mtot", &x[1]);
  reader->AddVariable("met", &x[2]);
  reader->AddVariable("m34", &x[3]);
  reader->AddVariable("pt34", &x[4]);
  reader->AddVariable("ylo", &x[5]);
  reader->AddVariable("yhi", &x[6]);
  reader->AddVariable("ctbz", &x[7]);
  reader->AddVariable("cts", &x[8]);
  reader->AddVariable("ctll", &x[9]);
  reader->AddVariable("mmin", &x[10]);
  reader->AddVariable("pvxp_n", &x[11]);

  std::string dir    = "../train/weights/";
  std::string prefix = "tth";
  reader->BookMVA("Fisher", dir + prefix + "_Fisher.weights.xml");

// Get the TTrees from the n-tuple files

  TFile* sigFile = new TFile("../train/tth_sig.root");
  TFile* bkgFile = new TFile("../train/tth_bkg.root");

  TTree* sig = (TTree*)sigFile->Get("mva");
  TTree* bkg = (TTree*)bkgFile->Get("mva");

  int nSig = sig->GetEntries();
  int nBkg = bkg->GetEntries();
  cout << "nSig = " << nSig << endl;
  cout << "nBkg = " << nBkg << endl;

  std::vector<TTree*> treeVec;
  treeVec.push_back(sig);
  treeVec.push_back(bkg);

// Loop over TTrees  (i.e., 0 = sig, 1 = bkg)

  for (int i=0; i<treeVec.size(); i++){

    treeVec[i]->Print();
    Event evt;
    treeVec[i]->SetBranchAddress("mvaInputs", &evt);
    int numEntries = treeVec[i]->GetEntries();

// Loop over events.  The test statistic is identified by the first 
// argument used above in BookMVA (e.g., "Fisher").

    for (int j=0; j<numEntries; j++){

      treeVec[i]->GetEntry(j);                // sets evt
      x[0] = evt.w;
      double w = x[0];                       // event weight
      x[1] = evt.mtot;                       // set variables of reader
      x[2] = evt.met;
      x[3] = evt.m34;
      x[4] = evt.pt34;
      x[5] = evt.ylo;
      x[6] = evt.yhi;
      x[7] = evt.ctbz;
      x[8] = evt.cts;
      x[9] = evt.ctll;
      x[10] = evt.mmin;
      x[11] = evt.pvxp_n;
      double tFisher = reader->EvaluateMVA("Fisher");

      if ( i == 0 ){ 
        hFishSig->Fill(tFisher, w);
      }
      else if ( i == 1 ){
        hFishBkg->Fill(tFisher, w);
      }

    }      // end of loop over events

  }          // end of loop over files

  histFile->Write();
  histFile->Close();

  return 0;

}
