#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <cstdlib>
#include <TFile.h>
#include <TChain.h>
#include <TList.h>
#include <TString.h>
#include <TMVA/Factory.h>
#include "Event.h"

using namespace std;

int main() {  

// Create ouput file, factory object and open the input file

  TFile* outputFile = TFile::Open( "TMVA.root", "RECREATE" );
  TMVA::Factory* factory = new TMVA::Factory("tth", outputFile, "");
  TFile* sigFile = new TFile("./tth_sig.root");
  TFile* bkgFile = new TFile("./tth_bkg.root");

// get the TTree objects from the input files

  TTree* sig = (TTree*)sigFile->Get("mva");
  TTree* bkg = (TTree*)bkgFile->Get("mva");
  int nSig = sig->GetEntries();
  int nBkg = bkg->GetEntries();

  cout << "nSig = " << nSig << endl;
  cout << "nBkg = " << nBkg << endl;

  double sigWeight = 1.0;        // global weight
  double bkgWeight = 1.0;
  factory->SetInputTrees(sig, bkg, sigWeight, bkgWeight);
  factory->SetWeightExpression("w");      // event-wise weights
   
// Define the input variables that shall be used for the MVA training
// (the variables used in the expression must exist in the original TTree).

  factory->AddVariable("mtot", 'F');
  factory->AddVariable("met", 'F');
  factory->AddVariable("m34", 'F');
  factory->AddVariable("pt34", 'F');
  factory->AddVariable("ylo", 'F');
  factory->AddVariable("yhi", 'F');
  factory->AddVariable("ctbz", 'F');
  factory->AddVariable("cts", 'F');
  factory->AddVariable("ctll", 'F');
  factory->AddVariable("mmin", 'F');
  factory->AddVariable("pvxp_n", 'F');

// Apply additional cuts on the signal and background sample.
// for example: TCut mycut = "abs(x)<0.5 && abs(y-0.5)<1";

  TCut mycut = "";

// 0 means use all; -1 means use none
// 0 for both tran and test means half of each 

  int nTrain_Signal = 0;
  int nTest_Signal = 0;
  int nTrain_Background = 0;
  int nTest_Background = 0;
  TString splitMode = "Block";        // default = "Random"
  factory->PrepareTrainingAndTestTree(mycut, nTrain_Signal, 
           nTrain_Background, nTest_Signal, nTest_Background, splitMode);

// Book MVA methods (see TMVA manual).  

  factory->BookMethod(TMVA::Types::kFisher, "Fisher", "H:!V:Fisher");   

// Train, test and evaluate all methods

  factory->TrainAllMethods();
  factory->TestAllMethods();
  factory->EvaluateAllMethods();    

// Save the output and finish up

  outputFile->Close();
  std::cout << "==> wrote root file TMVA.root" << std::endl;
  std::cout << "==> TMVAnalysis is done!" << std::endl; 

  delete factory;
  return 0;

}
